@@ -57,12 +57,10 @@ fileio_get_absolute_path (const gchar *file,
 {
 #ifdef G_OS_WIN32
   gchar *utf8str, *filename;
-  gint i = 0, j, s;
+  gint i = 0, j;
   DWORD dwSize;             /* ja:フルパスの長さ */
   HANDLE hFind;             /* ja:検索ハンドル */
-  LPTSTR p, lpszFile;
-  TCHAR szFile[MAX_PATH];   /* ja:フルパス */
-  TCHAR szLongName[MAX_PATH];
+  LPTSTR p, lpszFile, lpszTemp, lpszLongName;
 
   if (!file)
     return NULL;
@@ -77,8 +75,8 @@ fileio_get_absolute_path (const gchar *file,
 # endif /* not UNICODE */
   if (PathIsRelative (lpszFile))
     {
-      gint leng;
-      LPSTR lpszPath = NULL, lpszTemp;
+      int nLeng;
+      LPTSTR lpszPath = NULL;
 
       if (path)
         {
@@ -96,34 +94,52 @@ fileio_get_absolute_path (const gchar *file,
       if (!lpszPath || PathIsRelative (lpszPath))
         {
           g_free (lpszPath);
-          lpszPath = g_malloc (MAX_PATH * sizeof (TCHAR));
-          GetCurrentDirectory (MAX_PATH, lpszPath);
+          dwSize = GetCurrentDirectory (0, NULL);
+          lpszPath = g_malloc (dwSize * sizeof (TCHAR));
+          if (GetCurrentDirectory (dwSize, lpszPath) != dwSize - 1)
+            {
+              g_free (lpszPath);
+              g_free (lpszFile);
+              return g_strdup (file);
+            }
         }
-      leng = lstrlen (lpszPath);
-      if (leng > 0 && G_IS_DIR_SEPARATOR (lpszPath[leng - 1]))
-        lpszPath[--leng] = '\0';
-      lpszTemp = g_malloc ((leng + lstrlen (lpszFile) + 2) * sizeof (TCHAR));
+      PathRemoveBackslash (lpszPath);
+      nLeng = lstrlen (lpszPath);
+      lpszTemp = g_malloc ((nLeng + lstrlen (lpszFile) + 2) * sizeof (TCHAR));
       lstrcpy (lpszTemp, lpszPath);
-      lpszTemp[leng] = G_DIR_SEPARATOR;
-      lpszTemp[leng + 1] = '\0';
+      lpszTemp[nLeng] = G_DIR_SEPARATOR;
+      lpszTemp[nLeng + 1] = '\0';
       lstrcat (lpszTemp, lpszFile);
       g_free (lpszPath);
       g_free (lpszFile);
       lpszFile = lpszTemp;
     }
-  dwSize = GetFullPathName (lpszFile, MAX_PATH, szFile, &p);
+  dwSize = GetFullPathName (lpszFile, 0, NULL, &p);
+  if (dwSize == 0)
+    {
+      g_free (lpszFile);
+      return g_strdup (file);
+    }
+  lpszTemp = g_malloc (dwSize * sizeof (TCHAR));
+  if (GetFullPathName (lpszFile, dwSize, lpszTemp, &p) != dwSize - 1)
+    {
+      g_free (lpszTemp);
+      g_free (lpszFile);
+      return g_strdup (file);
+    }
+  dwSize--;
   g_free (lpszFile);
-  if (dwSize == 0 || dwSize >= MAX_PATH)
-    return g_strdup (file);
+  lpszFile = lpszTemp;
   if (dwSize >= 2)
     {
-      if (szFile[1] == ':')
+      if (lpszFile[1] == ':')
         {
-          szLongName[0] = (TCHAR)CharUpper ((LPTSTR)szFile[0]);
-          szLongName[1] = ':';
-          if (G_IS_DIR_SEPARATOR (szFile[2]))
+          lpszLongName = g_malloc (sizeof (TCHAR) * 4);
+          lpszLongName[0] = (TCHAR)CharUpper ((LPTSTR)lpszFile[0]);
+          lpszLongName[1] = ':';
+          if (G_IS_DIR_SEPARATOR (lpszFile[2]))
             {
-              szLongName[2] = G_DIR_SEPARATOR;
+              lpszLongName[2] = G_DIR_SEPARATOR;
               i = 3;
             }
           else
@@ -131,66 +147,78 @@ fileio_get_absolute_path (const gchar *file,
               i = 2;
             }
         }
-      else if (G_IS_DIR_SEPARATOR (szFile[0])
-                                            && G_IS_DIR_SEPARATOR (szFile[1]))
+      else if (G_IS_DIR_SEPARATOR (lpszFile[0])
+                                        && G_IS_DIR_SEPARATOR (lpszFile[1]))
         {
           i = 2;
           while (i < dwSize)
             {
-              if (G_IS_DIR_SEPARATOR (szFile[i]))
+              if (G_IS_DIR_SEPARATOR (lpszFile[i]))
                 break;
 # ifdef UNICODE
               i++;
 # else /* not UNICODE */
-              i += IsDBCSLeadByteEx (CP_ACP, szFile[i]) ? 2 : 1;
+              i += IsDBCSLeadByteEx (CP_ACP, lpszFile[i]) ? 2 : 1;
 # endif /* not UNICODE */
             }
           i++;
-          g_memmove (szLongName, szFile, i * sizeof (TCHAR));
+          lpszLongName = g_malloc ((i + 1) * sizeof (TCHAR));
+          g_memmove (lpszLongName, lpszFile, i * sizeof (TCHAR));
         }
     }
-  szLongName[i] = '\0';
+  lpszLongName[i] = '\0';
   j = i;
   while (i <= dwSize)
     {
-      if (szFile[i] == '\0' || G_IS_DIR_SEPARATOR (szFile[i]))
+      if (lpszFile[i] == '\0' || G_IS_DIR_SEPARATOR (lpszFile[i]))
         {
+          int nLeng;
           WIN32_FIND_DATA rcf;
 
-          s = lstrlen (szLongName);
-          g_memmove (szLongName + s, szFile + j, (i - j) * sizeof (TCHAR));
-          szLongName[s + i - j] = '\0';
-          hFind = FindFirstFile (szLongName, &rcf);
+          nLeng = lstrlen (lpszLongName);
+          lpszLongName = g_realloc (lpszLongName,
+                                        (nLeng + i - j + 2) * sizeof (TCHAR));
+          g_memmove (lpszLongName + nLeng, lpszFile + j,
+                                                    (i - j) * sizeof (TCHAR));
+          lpszLongName[nLeng + i - j] = '\0';
+          hFind = FindFirstFile (lpszLongName, &rcf);
           if (hFind != INVALID_HANDLE_VALUE)
             FindClose (hFind);
           else
             lstrcpy (rcf.cFileName, _T("."));
           if (lstrcmp (rcf.cFileName, _T(".")) != 0)
             {
-              g_memmove (szLongName + s, rcf.cFileName,
-                                    lstrlen (rcf.cFileName) * sizeof (TCHAR));
-              s += lstrlen (rcf.cFileName);
+              int nLen;
+
+              nLen = lstrlen (rcf.cFileName);
+              lpszLongName = g_realloc (lpszLongName,
+                                        (nLeng + nLen + 2) * sizeof (TCHAR));
+              g_memmove (lpszLongName + nLeng, rcf.cFileName,
+                                                        nLen * sizeof (TCHAR));
+              nLeng += nLen;
             }
           else
             {
-              s += i - j;
+              nLeng += i - j;
             }
-          if (G_IS_DIR_SEPARATOR (szFile[i]))
-            szLongName[s++] = G_DIR_SEPARATOR;
-          szLongName[s] = '\0';
+          if (G_IS_DIR_SEPARATOR (lpszFile[i]))
+            lpszLongName[nLeng++] = G_DIR_SEPARATOR;
+          lpszLongName[nLeng] = '\0';
           j = i + 1;
         }
 # ifdef UNICODE
       i++;
 # else /* not UNICODE */
-      i += IsDBCSLeadByteEx (CP_ACP, szFile[i]) ? 2 : 1;
+      i += IsDBCSLeadByteEx (CP_ACP, lpszFile[i]) ? 2 : 1;
 # endif /* not UNICODE */
     }
+  g_free (lpszFile);
 # ifdef UNICODE
-  utf8str = g_utf16_to_utf8 (szLongName, -1, NULL, NULL, NULL);
+  utf8str = g_utf16_to_utf8 (lpszLongName, -1, NULL, NULL, NULL);
 # else /* not UNICODE */
-  utf8str = g_locale_to_utf8 (szLongName, -1, NULL, NULL, NULL);
+  utf8str = g_locale_to_utf8 (lpszLongName, -1, NULL, NULL, NULL);
 # endif /* not UNICODE */
+  g_free (lpszLongName);
   if (!utf8str)
     return g_strdup (file);
   filename = g_filename_from_utf8 (utf8str, -1, NULL, NULL, NULL);
@@ -1274,7 +1302,8 @@ fileio_extract_icon (const gchar *file,
 #  else /* not UNICODE */
   lpszFile = g_win32_locale_filename_from_utf8 (file);
 #  endif /* not UNICODE */
-  if ((BOOL)SHGetFileInfo (lpszFile, 0, &shfi, sizeof (SHFILEINFOW),
+  shfi.dwAttributes = FILE_ATTRIBUTE_NORMAL;
+  if ((BOOL)SHGetFileInfo (lpszFile, 0, &shfi, sizeof (SHFILEINFO),
                                 SHGFI_ICON | SHGFI_USEFILEATTRIBUTES | uFlags)
                                             && GetIconInfo (shfi.hIcon, &ii))
     {
