@@ -712,10 +712,15 @@ misc_scroll_window (GtkWidget  *widget,
       cr = gdk_cairo_create (window);
       w = width - ABS (dx);
       h = height - ABS (dy);
+# if GTK_CHECK_VERSION(3,0,0)
+      pixbuf = gdk_pixbuf_get_from_window (window,
+                            dx > 0 ? x : x - dx, dy > 0 ? y : y - dy, w, h);
+# else /* not GTK_CHECK_VERSION(3,0,0) */
       pixbuf = gdk_pixbuf_get_from_drawable (NULL, window,
                                     gdk_colormap_get_system (),
                                     dx > 0 ? x : x - dx, dy > 0 ? y : y - dy,
                                     0, 0, w, h);
+# endif /* not GTK_CHECK_VERSION(3,0,0) */
       xx = dx > 0 ? x + dx : x;
       yy = dy > 0 ? y + dy : y;
       gdk_cairo_set_source_pixbuf (cr, pixbuf, xx, yy);
