@@ -1,10 +1,10 @@
 /*
     orzmdi
-    copyright (c) 1998-2007 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2012 Kazuki Iwamoto http://www.maid.org/ iwm@maid.org
 
-    This program is free software; you can redistribute it and/or modify
+    This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation, either version 3 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -13,18 +13,11 @@
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 #include "orzmdi.h"
 #include "misc/fileio.h"
-
-
-static void orz_mdi_class_init (OrzMdiClass *klass);
-static void orz_mdi_init       (OrzMdi      *mdi);
-
-
-static GtkNotebookClass *parent_class = NULL;
+#include "misc/misc.h"
 
 
 enum
@@ -37,8 +30,6 @@ enum
   SWITCH_WINDOW_SIGNAL,
   LAST_SIGNAL
 };
-
-
 enum
 {
   TARGET_ORZ_MDI_TAB
@@ -51,7 +42,7 @@ struct _OrzMdiTab
   gboolean created;     /* ja:TRUE:新規作成,FALSE:既存ファイル */
   gboolean edited;      /* ja:TRUE:編集済み,FALSE:未編集 */
   gchar *file;
-  gint same;            /* -2:名前のみ,-1:フルパス,0...:フルパス+数字 */
+  gint same;            /* ja:-2:名前のみ,-1:フルパス,0...:フルパス+数字 */
   GtkWidget *child, *image, *label;
   OrzMdi *mdi;
 };
@@ -66,34 +57,22 @@ static gint orz_mdi_signals[LAST_SIGNAL] = {0};
 static void
 orz_mdi_label_markup (OrzMdiTab *tab)
 {
-  gchar *label, *str;
+  gchar *label;
 
-  if (tab->same < -1)
-    {
-#if GLIB_CHECK_VERSION(2,6,0)
-      label = g_filename_display_basename (tab->file);
-#else /* not GLIB_CHECK_VERSION(2,6,0) */
-      str = g_path_get_basename (tab->file);
-      label = g_filename_to_utf8 (str, -1, NULL, NULL, NULL);
-      g_free (str);
-#endif /* not GLIB_CHECK_VERSION(2,6,0) */
-    }
-  else
-    {
-#if GLIB_CHECK_VERSION(2,6,0)
-      label = g_filename_display_name (tab->file);
-#else /* not GLIB_CHECK_VERSION(2,6,0) */
-      label = g_filename_to_utf8 (tab->file, -1, NULL, NULL, NULL);
-#endif /* not GLIB_CHECK_VERSION(2,6,0) */
-    }
+  label = tab->same < -1 ? g_filename_display_basename (tab->file)
+                         : g_filename_display_name (tab->file);
   if (tab->edited)
     {
+      gchar *str;
+
       str = g_strconcat ("<b><i>", label, "</i></b>", NULL);
       g_free (label);
       label = str;
     }
   if (tab->same >= 0)
     {
+      gchar *str;
+
       str = g_strdup_printf ("%s:%d", label, tab->same);
       g_free (label);
       label = str;
@@ -107,18 +86,17 @@ static void
 file_add_edit (OrzMdi    *mdi,
                OrzMdiTab *tab0)
 {
-  gint i;
-  GList *glist;
+  GList *gl, *glist;
   OrzMdiTab *tab1;
 
   tab0->same = -2;
   /* ja:ファイル名を調べる */
   glist = gtk_container_get_children (GTK_CONTAINER (mdi));
   /* ja:同名同パスを調べる */
-  for (i = g_list_length (glist) - 1; i >= 0; i--)
+  for (gl = g_list_first (glist); gl; gl = g_list_next (gl))
     {
       tab1 = g_object_get_data (G_OBJECT (gtk_notebook_get_tab_label
-            (GTK_NOTEBOOK (mdi), g_list_nth_data (glist, i))), "user_data");
+                                (GTK_NOTEBOOK (mdi), gl->data)), "user_data");
       if (tab1 && tab0 != tab1 && g_strfilecmp (tab0->file, tab1->file) == 0)
         {
           if (tab1->same < 0)
@@ -137,10 +115,10 @@ file_add_edit (OrzMdi    *mdi,
       /* ja:同名同パスはないとき */
       /* ja:同名異パスを調べる */
       name0 = g_path_get_basename (tab0->file);
-      for (i = g_list_length (glist) - 1; i >= 0; i--)
+      for (gl = g_list_first (glist); gl; gl = g_list_next (gl))
         {
           tab1 = g_object_get_data (G_OBJECT (gtk_notebook_get_tab_label
-            (GTK_NOTEBOOK (mdi), g_list_nth_data (glist, i))), "user_data");
+                                (GTK_NOTEBOOK (mdi), gl->data)), "user_data");
           if (tab1 && tab0 != tab1)
             {
               gchar *name1;
@@ -160,7 +138,7 @@ file_add_edit (OrzMdi    *mdi,
             }
         }
       g_free (name0);
-      if (i >= 0)
+      if (gl)
         tab0->same = -1;
     }
   g_list_free (glist);
@@ -172,18 +150,18 @@ file_remove_edit (OrzMdi      *mdi,
                   OrzMdiTab   *tab0,
                   const gchar *file)
 {
-  gint i, count0 = 0, count1 = 0;
+  gint count0 = 0, count1 = 0;
   gchar *name0, *name1;
-  GList *glist;
+  GList *gl, *glist;
   OrzMdiTab *tab1;
 
   /* ja:ファイル名を調べる */
   glist = gtk_container_get_children (GTK_CONTAINER (mdi));
   name0 = g_path_get_basename (tab0->file);
-  for (i = g_list_length (glist) - 1; i >= 0; i--)
+  for (gl = g_list_first (glist); gl; gl = g_list_next (gl))
     {
       tab1 = g_object_get_data (G_OBJECT (gtk_notebook_get_tab_label
-            (GTK_NOTEBOOK (mdi), g_list_nth_data (glist, i))), "user_data");
+                                (GTK_NOTEBOOK (mdi), gl->data)), "user_data");
       if (tab1 && tab0 != tab1)
         {
           /* ja:同名同パスを調べる */
@@ -203,12 +181,11 @@ file_remove_edit (OrzMdi      *mdi,
       name2 = file ? g_path_get_basename (file) : NULL;
       if (!name2 || g_strfilecmp (name0, name2) != 0)
         /* 名前が変わっているとき */
-        for (i = g_list_length (glist) - 1; i >= 0; i--)
+        for (gl = g_list_first (glist); gl; gl = g_list_next (gl))
           {
             /* ja:同名のファイルを通常表示に変更する */
             tab1 = g_object_get_data (G_OBJECT (gtk_notebook_get_tab_label
-                            (GTK_NOTEBOOK (mdi), g_list_nth_data (glist, i))),
-                                                                "user_data");
+                                (GTK_NOTEBOOK (mdi), gl->data)), "user_data");
             if (tab1 && tab0 != tab1)
               {
                 /* ja:同名を調べる */
@@ -224,18 +201,21 @@ file_remove_edit (OrzMdi      *mdi,
       g_free (name2);
     }
   else if (count0 == 1)/* ja:同名同パスが1つだけある(異パスは複数)とき */
-    for (i = g_list_length (glist) - 1; i >= 0; i--)
-      {
-        /* ja:同名同パスのファイルをフルパスに変更する */
-        tab1 = g_object_get_data (G_OBJECT (gtk_notebook_get_tab_label
-            (GTK_NOTEBOOK (mdi), g_list_nth_data (glist, i))), "user_data");
-        /* ja:同名同パスを調べる */
-        if (tab1 && tab0 != tab1 && g_strfilecmp (tab0->file, tab1->file) == 0)
-          {
-            tab1->same = -1;
-            orz_mdi_label_markup (tab1);
-          }
-      }
+    {
+      for (gl = g_list_first (glist); gl; gl = g_list_next (gl))
+        {
+          /* ja:同名同パスのファイルをフルパスに変更する */
+          tab1 = g_object_get_data (G_OBJECT (gtk_notebook_get_tab_label
+                                (GTK_NOTEBOOK (mdi), gl->data)), "user_data");
+          /* ja:同名同パスを調べる */
+          if (tab1 && tab0 != tab1
+                                && g_strfilecmp (tab0->file, tab1->file) == 0)
+            {
+              tab1->same = -1;
+              orz_mdi_label_markup (tab1);
+            }
+        }
+    }
   g_free (name0);
   g_list_free (glist);
 }
@@ -244,32 +224,7 @@ file_remove_edit (OrzMdi      *mdi,
 /******************************************************************************
 *                                                                             *
 ******************************************************************************/
-GtkType
-orz_mdi_get_type (void)
-{
-  static GType orz_mdi_type = 0;
-
-  if (!orz_mdi_type)
-    {
-      const static GTypeInfo mdi_info =
-      {
-        sizeof (OrzMdiClass),
-        NULL,               /* base_init */
-        NULL,               /* base_finalize */
-        (GClassInitFunc)orz_mdi_class_init,
-        NULL,               /* class_finalize */
-        NULL,               /* class_data */
-        sizeof (OrzMdi),
-        0,                  /* n_preallocs */
-        (GInstanceInitFunc)orz_mdi_init,
-      };
-
-      orz_mdi_type = g_type_register_static (GTK_TYPE_NOTEBOOK, "OrzMdi",
-                                                                &mdi_info, 0);
-    }
-
-  return orz_mdi_type;
-}
+G_DEFINE_TYPE (OrzMdi, orz_mdi, GTK_TYPE_NOTEBOOK);
 
 
 /* ja:子ウィジェットが破棄されるとき */
@@ -293,23 +248,25 @@ orz_mdi_remove (GtkContainer *container,
                     gtk_notebook_page_num (GTK_NOTEBOOK (container), widget),
                     tab->page_data);
           file_remove_edit (ORZ_MDI (container), tab, NULL);
-          g_free (tab->file);
-          g_free (tab);
-          g_object_set_data (G_OBJECT (tab_label), "user_data", NULL);
         }
     }
-  ((GtkContainerClass *)parent_class)->remove (container, widget);
+  GTK_CONTAINER_CLASS (orz_mdi_parent_class)->remove (container, widget);
 }
 
 
 static void
 orz_mdi_switch_page (GtkNotebook     *notebook,
+#if GTK_CHECK_VERSION(2,22,0)
+                     gpointer         page,
+#else /* not GTK_CHECK_VERSION(2,22,0) */
                      GtkNotebookPage *page,
+#endif /* not GTK_CHECK_VERSION(2,22,0) */
                      guint            page_num)
 {
   GtkWidget *child;
 
-  parent_class->switch_page (notebook, page, page_num);
+  GTK_NOTEBOOK_CLASS (orz_mdi_parent_class)->switch_page
+                                                    (notebook, page, page_num);
   child = gtk_notebook_get_nth_page (notebook, page_num);
   if (child)
     {
@@ -338,11 +295,9 @@ orz_mdi_class_init (OrzMdiClass *klass)
   GtkContainerClass *container_class;
   GtkNotebookClass *notebook_class;
 
-  container_class = (GtkContainerClass *) klass;
-  notebook_class = (GtkNotebookClass *) klass;
-
-  parent_class = g_type_class_peek_parent (klass);
+  container_class = GTK_CONTAINER_CLASS (klass);
   container_class->remove = orz_mdi_remove;
+  notebook_class = GTK_NOTEBOOK_CLASS (klass);
   notebook_class->switch_page = orz_mdi_switch_page;
 
   klass->add_window = NULL;
@@ -418,7 +373,9 @@ orz_mdi_class_init (OrzMdiClass *klass)
 static void
 orz_mdi_init (OrzMdi *mdi)
 {
+#if ! GTK_CHECK_VERSION(2,12,0)
   mdi->tooltips = gtk_tooltips_new ();
+#endif /* not GTK_CHECK_VERSION(2,12,0) */
 }
 
 
@@ -455,8 +412,8 @@ orz_mdi_clicked (GtkWidget *widget,
 
 /* ja:子ウィジェットが破棄されるとき */
 static void
-orz_mdi_destroy (GtkWidget *widget,
-                 OrzMdi    *mdi)
+orz_mdi_destroy_child (GtkWidget *widget,
+                       OrzMdi    *mdi)
 {
   GtkWidget *tab_label;
 
@@ -474,14 +431,28 @@ orz_mdi_destroy (GtkWidget *widget,
                          gtk_notebook_page_num (GTK_NOTEBOOK (mdi), widget),
                          tab->page_data);
           file_remove_edit (ORZ_MDI (mdi), tab, NULL);
-          g_free (tab->file);
-          g_free (tab);
-          g_object_set_data (G_OBJECT (tab_label), "user_data", NULL);
         }
     }
 }
 
 
+/* ja:タブが破棄されるとき */
+static void
+orz_mdi_destroy_tab (GtkWidget *widget,
+                     OrzMdi    *mdi)
+{
+  OrzMdiTab *tab;
+
+  tab = g_object_get_data (G_OBJECT (widget), "user_data");
+  if (tab)
+    {
+      g_free (tab->file);
+      g_free (tab);
+      g_object_set_data (G_OBJECT (widget), "user_data", NULL);
+    }
+}
+
+
 /* ja:ドロップされたとき */
 static void
 orz_mdi_tab_drag_begin (GtkWidget      *widget,
@@ -510,7 +481,7 @@ orz_mdi_tab_drag_data_get (GtkWidget        *widget,
   const static gchar *text = "dummy";
 
   if (info == TARGET_ORZ_MDI_TAB)
-    gtk_selection_data_set (data, data->target, 8,
+    gtk_selection_data_set (data, gtk_selection_data_get_selection (data), 8,
                                         (const guchar *)text, g_strlen (text));
 }
 static void
@@ -543,29 +514,67 @@ orz_mdi_tab_drag_data_received (GtkWidget        *widget,
 }
 
 
-/*  ja:ページのタブを作る
+/*  ja:ページを最後に追加する
           mdi,ウィジェット
         child,内容
          file,ファイル名
       created,TRUE:新規作成,FALSE:既存ファイル
     page_data,ページデータ
-          RET,ウィジェット                                                  */
-static GtkWidget *
-orz_mdi_create_tab_label (OrzMdi         *mdi,
-                          GtkWidget      *child,
-                          const gchar    *file,
-                          const gboolean  created,
-                          gpointer        page_data)
+          RET,ページ(0...,-1:エラー)                                        */
+gint
+orz_mdi_append_page (OrzMdi         *mdi,
+                     GtkWidget      *child,
+                     const gchar    *file,
+                     const gboolean  created,
+                     gpointer        page_data)
 {
+  return orz_mdi_insert_page (mdi, child, -1, file, created, page_data);
+}
+
+
+/*  ja:ページを最初に追加する
+          mdi,ウィジェット
+        child,内容
+         file,ファイル名
+      created,TRUE:新規作成,FALSE:既存ファイル
+    page_data,ページデータ
+          RET,ページ(0...,-1:エラー)                                        */
+gint
+orz_mdi_prepend_page (OrzMdi         *mdi,
+                      GtkWidget      *child,
+                      const gchar    *file,
+                      const gboolean  created,
+                      gpointer        page_data)
+{
+  return orz_mdi_insert_page (mdi, child, 0, file, created, page_data);
+}
+
+
+/*  ja:ページを挿入する
+          mdi,ウィジェット
+        child,内容
+     position,位置(0...)
+         file,ファイル名
+      created,TRUE:新規作成,FALSE:既存ファイル
+    page_data,ページデータ
+       RET,ページ(0...,-1:エラー)                                           */
+gint
+orz_mdi_insert_page (OrzMdi         *mdi,
+                     GtkWidget      *child,
+                     const gint      position,
+                     const gchar    *file,
+                     const gboolean  created,
+                     gpointer        page_data)
+{
+  gint page_num;
   GdkPixbuf *pixbuf;
-  GtkIconSet *icon_set;
-  GtkWidget *image = NULL, *button, *hbox, *event;
   GtkTargetEntry dnd_entries[1] = {{"ORZ_MDI_TAB",
                                     GTK_TARGET_SAME_APP, TARGET_ORZ_MDI_TAB}};
+  GtkWidget *button, *hbox, *event;
   OrzMdiTab *tab;
 
-  if (!mdi)
-    return NULL;
+  if (!mdi || !child)
+    return -1;
   tab = g_malloc0 (sizeof (OrzMdiTab));
   tab->page_data = page_data;
   tab->created = created;
@@ -575,108 +584,9 @@ orz_mdi_create_tab_label (OrzMdi         *mdi,
   tab->mdi = mdi;
   tab->child = child;
   file_add_edit (mdi, tab);
-  /* ja:イメージ */
-  icon_set = gtk_icon_factory_lookup_default (GTK_STOCK_CLOSE);
-  if (icon_set)
-    {
-      pixbuf = gtk_icon_set_render_icon (icon_set,
-                                         gtk_widget_get_default_style (),
-                                         GTK_TEXT_DIR_NONE,
-                                         GTK_STATE_NORMAL,
-                                         GTK_ICON_SIZE_MENU,
-                                         NULL, NULL);
-      if (pixbuf)
-        {
-          if (gdk_pixbuf_get_colorspace (pixbuf) == GDK_COLORSPACE_RGB
-                            && gdk_pixbuf_get_n_channels (pixbuf) == 4
-                            && gdk_pixbuf_get_has_alpha (pixbuf)
-                            && gdk_pixbuf_get_bits_per_sample (pixbuf) == 8)
-            {
-              int width, height, rowstride;
-              gint x, y, left, top, right, bottom;
-              guchar *pixels;
-
-              pixels = gdk_pixbuf_get_pixels (pixbuf);
-              width = gdk_pixbuf_get_width (pixbuf);
-              height = gdk_pixbuf_get_height (pixbuf);
-              rowstride = gdk_pixbuf_get_rowstride (pixbuf);
-              for (x = 0; x < width; x++)
-                for (y = 0; y < height; y++)
-                  if (pixels[y * rowstride + x * 4 + 3] > 0)
-                    goto loop_left;
-              loop_left: left = x;
-              for (y = 0; y < height; y++)
-                for (x = 0; x < width; x++)
-                  if (pixels[y * rowstride + x * 4 + 3] > 0)
-                    goto loop_top;
-              loop_top: top = y;
-              for (x = width - 1; x >= 0; x--)
-                for (y = 0; y < height; y++)
-                  if (pixels[y * rowstride + x * 4 + 3] > 0)
-                    goto loop_right;
-              loop_right: right = x;
-              for (y = height - 1; y >= 0; y--)
-                for (x = 0; x < width; x++)
-                  if (pixels[y * rowstride + x * 4 + 3] > 0)
-                    goto loop_bottom;
-              loop_bottom: bottom = y;
-              if (left <= right && top <= bottom)
-                {
-                  int w, h, r;
-                  guchar *p;
-                  gint i;
-                  GdkPixbuf *b;
-
-                  w = right - left + 1;
-                  h = bottom - top + 1;
-                  b = gdk_pixbuf_new (GDK_COLORSPACE_RGB, TRUE, 8, w, h);
-                  p = gdk_pixbuf_get_pixels (b);
-                  r = gdk_pixbuf_get_rowstride (b);
-                  for (x = 0; x < w; x++)
-                    for (y = 0; y < h; y++)
-                      for (i = 0; i < 4; i++)
-                        p[y * r + x * 4 + i]
-                        = pixels[(y + top) * rowstride + (x + left) * 4 + i];
-                  image = gtk_image_new_from_pixbuf (b);
-                  g_object_unref (b);
-                }
-            }
-          g_object_unref (pixbuf);
-        }
-    }
-  if (!image)
-    image = gtk_image_new_from_stock (GTK_STOCK_CLOSE, GTK_ICON_SIZE_MENU);
-  if (!image)
-    {
-      const static gchar *xpm[] = {
-"12 13 4 1",
-"   c None",
-"X  c #000000",
-"+  c #808080",
-".  c #FFFFFF",
-"+++++++++++.",
-"+.........+.",
-"+.        +.",
-"+.XX    XX+.",
-"+. XX  XX +.",
-"+.  XXXX  +.",
-"+.   XX   +.",
-"+.  XXXX  +.",
-"+. XX  XX +.",
-"+.XX    XX+.",
-"+.        +.",
-"+++++++++++.",
-"............"};
-
-      pixbuf = gdk_pixbuf_new_from_xpm_data (xpm);
-      image = gtk_image_new_from_pixbuf (pixbuf);
-      g_object_unref (pixbuf);
-    }
   /* ja:ボタン */
-  button = gtk_button_new ();
-  gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
-  gtk_container_add (GTK_CONTAINER (button), image);
-  gtk_widget_show_all (button);
+  button = misc_close_button ();
+  gtk_widget_show (button);
   /* ja:ラベル */
   tab->label = gtk_label_new ("");
   orz_mdi_label_markup (tab);
@@ -698,163 +608,42 @@ orz_mdi_create_tab_label (OrzMdi         *mdi,
   gtk_widget_show (hbox);
   /* ja:ツールチップス */
   event = gtk_event_box_new ();
+#if GLIB_CHECK_VERSION(2,4,0)
+  gtk_event_box_set_visible_window (GTK_EVENT_BOX (event), FALSE);
+#endif /* GLIB_CHECK_VERSION(2,4,0) */
   gtk_container_add (GTK_CONTAINER (event), hbox);
+#if GTK_CHECK_VERSION(2,12,0)
+  gtk_widget_set_has_tooltip (event, TRUE);
+  gtk_widget_set_tooltip_text (event, tab->file);
+#else /* not GTK_CHECK_VERSION(2,12,0) */
   gtk_tooltips_set_tip (mdi->tooltips, event, tab->file, tab->file);
+#endif /* not GTK_CHECK_VERSION(2,12,0) */
   g_object_set_data (G_OBJECT (event), "user_data", tab);
 
   g_signal_connect (G_OBJECT (button), "clicked",
-                                        G_CALLBACK (orz_mdi_clicked), event);
+                                    G_CALLBACK (orz_mdi_clicked), event);
   g_signal_connect (G_OBJECT (child), "destroy",
-                                        G_CALLBACK (orz_mdi_destroy), mdi);
+                                    G_CALLBACK (orz_mdi_destroy_child), mdi);
+  g_signal_connect (G_OBJECT (event), "destroy",
+                                    G_CALLBACK (orz_mdi_destroy_tab), mdi);
   /* en:Drag & Drop */
-  gtk_drag_source_set (event, GDK_BUTTON1_MASK, dnd_entries, 1,
-                                                            GDK_ACTION_MOVE);
-  gtk_drag_dest_set (event, GTK_DEST_DEFAULT_ALL, dnd_entries, 1,
-                                                            GDK_ACTION_MOVE);
+  gtk_drag_source_set (event, GDK_BUTTON1_MASK,
+                    dnd_entries, G_N_ELEMENTS (dnd_entries), GDK_ACTION_MOVE);
+  gtk_drag_dest_set (event, GTK_DEST_DEFAULT_ALL,
+                    dnd_entries, G_N_ELEMENTS (dnd_entries), GDK_ACTION_MOVE);
   g_signal_connect (G_OBJECT (event), "drag_begin",
                             G_CALLBACK (orz_mdi_tab_drag_begin), NULL);
   g_signal_connect (G_OBJECT (event), "drag_data_get",
                             G_CALLBACK (orz_mdi_tab_drag_data_get), NULL);
   g_signal_connect (G_OBJECT (event), "drag_data_received",
                             G_CALLBACK (orz_mdi_tab_drag_data_received), NULL);
-
-  return event;
-}
-
-
-/*  ja:ページを最後に追加する
-          mdi,ウィジェット
-        child,内容
-         file,ファイル名
-      created,TRUE:新規作成,FALSE:既存ファイル
-    page_data,ページデータ
-          RET,ページ(0...,-1:エラー)                                        */
-gint
-orz_mdi_append_page (OrzMdi         *mdi,
-                     GtkWidget      *child,
-                     const gchar    *file,
-                     const gboolean  created,
-                     gpointer        page_data)
-{
-  return orz_mdi_insert_page_menu (mdi, child, NULL, -1,
-                                                    file, created, page_data);
-}
-
-
-/*  ja:ページを最後に追加する
-          mdi,ウィジェット
-        child,内容
-         menu,メニュー
-         file,ファイル名
-      created,TRUE:新規作成,FALSE:既存ファイル
-    page_data,ページデータ
-          RET,ページ(0...,-1:エラー)                                        */
-gint
-orz_mdi_append_page_menu (OrzMdi         *mdi,
-                          GtkWidget      *child,
-                          GtkWidget      *menu,
-                          const gchar    *file,
-                          const gboolean  created,
-                          gpointer        page_data)
-{
-  return orz_mdi_insert_page_menu (mdi, child, menu, -1,
-                                                    file, created, page_data);
-}
-
-
-/*  ja:ページを最初に追加する
-          mdi,ウィジェット
-        child,内容
-         file,ファイル名
-      created,TRUE:新規作成,FALSE:既存ファイル
-    page_data,ページデータ
-          RET,ページ(0...,-1:エラー)                                        */
-gint
-orz_mdi_prepend_page (OrzMdi         *mdi,
-                      GtkWidget      *child,
-                      const gchar    *file,
-                      const gboolean  created,
-                      gpointer        page_data)
-{
-  return orz_mdi_insert_page_menu (mdi, child, NULL, 0,
-                                                    file, created, page_data);
-}
-
-
-/*  ja:ページを最初に追加する
-          mdi,ウィジェット
-        child,内容
-         menu,メニュー
-         file,ファイル名
-      created,TRUE:新規作成,FALSE:既存ファイル
-    page_data,ページデータ
-          RET,ページ(0...,-1:エラー)                                        */
-gint
-orz_mdi_prepend_page_menu (OrzMdi         *mdi,
-                           GtkWidget      *child,
-                           GtkWidget      *menu,
-                           const gchar    *file,
-                           const gboolean  created,
-                           gpointer        page_data)
-{
-  return orz_mdi_insert_page_menu (mdi, child, menu, 0,
-                                                    file, created, page_data);
-}
-
-
-/*  ja:ページを挿入する
-          mdi,ウィジェット
-        child,内容
-     position,位置(0...)
-         file,ファイル名
-      created,TRUE:新規作成,FALSE:既存ファイル
-    page_data,ページデータ
-       RET,ページ(0...,-1:エラー)                                           */
-gint
-orz_mdi_insert_page (OrzMdi         *mdi,
-                     GtkWidget      *child,
-                     const gint      position,
-                     const gchar    *file,
-                     const gboolean  created,
-                     gpointer        page_data)
-{
-  return orz_mdi_insert_page_menu (mdi, child, NULL, position,
-                                                    file, created, page_data);
-}
-
-
-/*  ja:ページを挿入する
-          mdi,ウィジェット
-        child,内容
-         menu,メニュー
-     position,位置(0...)
-         file,ファイル名
-      created,TRUE:新規作成,FALSE:既存ファイル
-    page_data,ページデータ
-       RET,ページ(0...,-1:エラー)                                           */
-gint
-orz_mdi_insert_page_menu (OrzMdi         *mdi,
-                          GtkWidget      *child,
-                          GtkWidget      *menu,
-                          const gint      position,
-                          const gchar    *file,
-                          const gboolean  created,
-                          gpointer        page_data)
-{
-  gint page_num = -1;
-  GtkWidget *tab_label;
-
-  tab_label = orz_mdi_create_tab_label (mdi, child, file, created, page_data);
-  if (tab_label)
-    {
-      page_num = gtk_notebook_insert_page_menu (GTK_NOTEBOOK (mdi), child,
-                                                    tab_label, menu, position);
-      g_signal_emit (mdi,
-                     orz_mdi_signals[ADD_WINDOW_SIGNAL],
-                     0,
-                     page_num,
-                     page_data);
-    }
+  page_num = gtk_notebook_insert_page (GTK_NOTEBOOK (mdi), child,
+                                                            event, position);
+  g_signal_emit (mdi,
+                 orz_mdi_signals[ADD_WINDOW_SIGNAL],
+                 0,
+                 page_num,
+                 page_data);
   return page_num;
 }
 
@@ -1025,8 +814,12 @@ orz_mdi_set_file (OrzMdi      *mdi,
                   file_add_edit (mdi, tab);
                   orz_mdi_label_markup (tab);
                   /* ja:ツールチップス */
+# if GTK_CHECK_VERSION(2,12,0)
+                  gtk_widget_set_tooltip_text (tab_label, tab->file);
+# else /* not GTK_CHECK_VERSION(2,12,0) */
                   gtk_tooltips_set_tip (mdi->tooltips, tab_label,
                                                         tab->file, tab->file);
+# endif /* not GTK_CHECK_VERSION(2,12,0) */
                   /* ja:アイコン */
                   hbox = gtk_bin_get_child (GTK_BIN (tab_label));
                   if (tab->image)
