@@ -1,6 +1,6 @@
 /*
     Melody maid
-    copyright (c) 1998-2011 Kazuki Iwamoto http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2013 Kazuki Iwamoto http://www.maid.org/ iwm@maid.org
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -289,31 +289,31 @@ add_dialog (MmaidWindow *mmaid)
   if (glist_title)
     {
       gtk_combo_set_popdown_strings (GTK_COMBO (combo_title), glist_title);
-      g_list_find_custom (glist_title, (GFunc)g_free, NULL);
+      g_list_foreach (glist_title, (GFunc)g_free, NULL);
       g_list_free (glist_title);
     }
   if (glist_artist)
     {
       gtk_combo_set_popdown_strings (GTK_COMBO (combo_artist), glist_artist);
-      g_list_find_custom (glist_artist, (GFunc)g_free, NULL);
+      g_list_foreach (glist_artist, (GFunc)g_free, NULL);
       g_list_free (glist_artist);
     }
   if (glist_album)
     {
       gtk_combo_set_popdown_strings (GTK_COMBO (combo_album), glist_album);
-      g_list_find_custom (glist_album, (GFunc)g_free, NULL);
+      g_list_foreach (glist_album, (GFunc)g_free, NULL);
       g_list_free (glist_album);
     }
   if (glist_comment)
     {
       gtk_combo_set_popdown_strings (GTK_COMBO (combo_comment), glist_comment);
-      g_list_find_custom (glist_comment, (GFunc)g_free, NULL);
+      g_list_foreach (glist_comment, (GFunc)g_free, NULL);
       g_list_free (glist_comment);
     }
   if (glist_genre)
     {
       gtk_combo_set_popdown_strings (GTK_COMBO (combo_genre), glist_genre);
-      g_list_find_custom (glist_genre, (GFunc)g_free, NULL);
+      g_list_foreach (glist_genre, (GFunc)g_free, NULL);
       g_list_free (glist_genre);
     }
   gtk_combo_set_value_in_list (GTK_COMBO (combo_title), FALSE, FALSE);
