@@ -1,6 +1,6 @@
 /*
     Melody maid
-    copyright (c) 1998-2011 Kazuki Iwamoto http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2012 Kazuki Iwamoto http://www.maid.org/ iwm@maid.org
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -60,7 +60,7 @@ void
 command_open (GtkWidget *widget,
               gpointer   user_data)
 {
-  gchar *file = NULL, *path, *title;
+  gchar *file = NULL, *title;
   GtkStockItem stock_item;
   GtkWidget *dialog;
 
@@ -85,6 +85,8 @@ command_open (GtkWidget *widget,
   dialog = gtk_file_selection_new (title);
   if (open_path)
     {
+      gchar *path;
+
       path = g_strconcat (open_path, G_SEARCHPATH_SEPARATOR_S, NULL);
       gtk_file_selection_set_filename (GTK_FILE_SELECTION (dialog), path);
       g_free (path);
@@ -98,6 +100,8 @@ command_open (GtkWidget *widget,
   gtk_widget_destroy (dialog);
   if (file)
     {
+      gchar *path;
+
       g_free (open_path);
       path = fileio_get_full_path (file);
       open_path = g_path_get_dirname (path);
@@ -176,32 +180,57 @@ command_save (GtkWidget *widget,
           result = gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_YES;
           gtk_widget_destroy (dialog);
         }
-      if (result &&  (mmaid->filekind == FILE_KIND_DB
-                        ? file_save_db (file, mmaid, root, mmaid->charset)
-                        : file_save_list (file, mmaid, root, mmaid->charset,
-                                                            mmaid->playlist))
-                && (mmaid->filekind == FILE_KIND_DB || mmaid->playlist != 2))
+      if (result &&  (mmaid->filetype == FILE_TYPE_DB
+            ? file_save_db (file, mmaid, root, mmaid->charset)
+            : file_save_list (file, mmaid, mmaid->charset,
+                mmaid->playlist, mmaid->separator, mmaid->crlf, mmaid->header))
+                 && (mmaid->filetype == FILE_TYPE_DB
+                                    || mmaid->playlist != PLAYLIST_BASENAME))
         orz_mdi_set_edited (ORZ_MDI (mdi), page_num, FALSE);
     }
   g_free (root);
 }
 
 
+typedef struct _SaveDialog
+{
+  GtkWidget *radio_db, *radio_plist;
+  GtkWidget *frame_path, *radio_relative, *radio_absolute, *radio_basename;
+  GtkWidget *frame_sep, *radio_slash, *radio_yen;
+  GtkWidget *check_header;
+  GtkWidget *combo_encoding, *radio_crlf, *radio_cr, *radio_lf;
+  GtkWidget *label_root, *combo_root;
+} SaveDialog;
+
+
 /* ja:ラジオボタンが変更された */
 static void
-command_save_toggled_radio0 (GtkWidget *widget,
-                             GtkWidget *hbox)
+command_save_toggled_radio_db (GtkWidget  *widget,
+                               SaveDialog *savedlg)
 {
-  gtk_widget_set_sensitive (hbox, FALSE);
+  gboolean active;
+
+  active = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget));
+  gtk_widget_set_sensitive (savedlg->frame_path, !active);
+  gtk_widget_set_sensitive (savedlg->frame_sep, !active
+                            && !gtk_toggle_button_get_active
+                                (GTK_TOGGLE_BUTTON (savedlg->radio_basename)));
+  gtk_widget_set_sensitive (savedlg->radio_crlf, !active);
+  gtk_widget_set_sensitive (savedlg->radio_cr, !active);
+  gtk_widget_set_sensitive (savedlg->radio_lf, !active);
+  gtk_widget_set_sensitive (savedlg->check_header, !active);
+  gtk_widget_set_sensitive (savedlg->label_root, active);
+  gtk_widget_set_sensitive (savedlg->combo_root, active);
 }
 
 
 /* ja:ラジオボタンが変更された */
 static void
-command_save_toggled_radio1 (GtkWidget *widget,
-                             GtkWidget *hbox)
+command_save_toggled_radio_basename (GtkWidget  *widget,
+                                     SaveDialog *savedlg)
 {
-  gtk_widget_set_sensitive (hbox, TRUE);
+  gtk_widget_set_sensitive (savedlg->frame_sep,
+                !gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)));
 }
 
 
@@ -262,15 +291,16 @@ void
 command_saveas (GtkWidget *widget,
                 gpointer   user_data)
 {
-  gchar *file, *root = NULL, *charset, *name, *path, *tmp, **ary, *title;
-  gint i, playlist, page_num;
-  guint filekind;
+  gboolean header;
+  gchar *file, *root = NULL, *charset, *path, **ary, *title;
+  gint i, playlist, separator, crlf, page_num;
+  guint filetype;
   GList *gl, *glist = NULL;
   GtkStockItem stock_item;
-  GtkWidget *dialog, *hbox0, *hbox1, *vbox;
-  GtkWidget *combo0, *combo1, *label0, *label1;
-  GtkWidget *radio0, *radio1, *radio2, *radio3, *radio4;
+  GtkWidget *dialog, *frame, *hbox, *vbox;
+  GtkWidget *label_path, *label_sep, *label_encoding;
   MmaidWindow *mmaid;
+  SaveDialog savedlg;
 
   page_num = gtk_notebook_get_current_page (GTK_NOTEBOOK (mdi));
   mmaid = orz_mdi_get_data (ORZ_MDI (mdi), page_num);
@@ -309,15 +339,15 @@ command_saveas (GtkWidget *widget,
     }
   *path = '\0';
 #ifdef G_OS_WIN32
-  if (g_strlen (root) > 2
-            && G_IS_DIR_SEPARATOR (root[0]) && G_IS_DIR_SEPARATOR (root[1]))
+  if (G_IS_DIR_SEPARATOR (root[0]) && G_IS_DIR_SEPARATOR (root[1]))
     {
-      path = g_strdup (G_DIR_SEPARATOR_S G_DIR_SEPARATOR_S);
       ary = g_strsplit (root + 2, G_DIR_SEPARATOR_S, 0);
+      path = g_strdup (G_DIR_SEPARATOR_S G_DIR_SEPARATOR_S);
     }
   else
     {
       ary = g_strsplit (root, G_DIR_SEPARATOR_S, 0);
+      path = NULL;
     }
 #else /* not G_OS_WIN32 */
   ary = g_strsplit (root, G_DIR_SEPARATOR_S, 0);
@@ -361,118 +391,175 @@ command_saveas (GtkWidget *widget,
   dialog = gtk_file_selection_new (title);
 #endif /* not GTK_CHECK_VERSION(2,4,0) */
   g_free (title);
+  /* ja:チェックボタン */
+  savedlg.check_header = gtk_check_button_new_with_mnemonic (_("_Header"));
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (savedlg.check_header),
+                                                                mmaid->header);
   /* ja:ラジオボタン */
-  radio0 = gtk_radio_button_new_with_mnemonic (NULL, _("_iDB File"));
-  radio1 = gtk_radio_button_new_with_mnemonic_from_widget
-                                (GTK_RADIO_BUTTON (radio0), _("_Play List"));
-  radio2 = gtk_radio_button_new_with_mnemonic (NULL, _("Rela_tive Path"));
-  radio3 = gtk_radio_button_new_with_mnemonic_from_widget
-                            (GTK_RADIO_BUTTON (radio2), _("Ab_solute Path"));
-  radio4 = gtk_radio_button_new_with_mnemonic_from_widget
-                            (GTK_RADIO_BUTTON (radio3), _("File Name _Only"));
-  hbox0 = gtk_hbox_new (FALSE, 0);
-  if (mmaid->filekind == FILE_KIND_DB)
-    {
-      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radio0), TRUE);
-      gtk_widget_set_sensitive (hbox0, FALSE);
-    }
-  else
-    {
-      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radio1), TRUE);
-    }
+  savedlg.radio_db = gtk_radio_button_new_with_mnemonic (NULL, _("_iDB File"));
+  savedlg.radio_plist = gtk_radio_button_new_with_mnemonic_from_widget
+                    (GTK_RADIO_BUTTON (savedlg.radio_db), _("_Play List"));
+  savedlg.radio_relative = gtk_radio_button_new_with_label
+                    (NULL, _("Relative"));
+  savedlg.radio_absolute = gtk_radio_button_new_with_label_from_widget
+                    (GTK_RADIO_BUTTON (savedlg.radio_relative), _("Absolute"));
+  savedlg.radio_basename = gtk_radio_button_new_with_label_from_widget
+                    (GTK_RADIO_BUTTON (savedlg.radio_absolute), _("File Name"));
+
+  savedlg.radio_slash = gtk_radio_button_new_with_label (NULL, "/");
+  savedlg.radio_yen = gtk_radio_button_new_with_label_from_widget
+                    (GTK_RADIO_BUTTON (savedlg.radio_slash), "\\");
+  savedlg.radio_crlf = gtk_radio_button_new_with_label (NULL, "CR+LF");
+  savedlg.radio_cr = gtk_radio_button_new_with_label_from_widget
+                    (GTK_RADIO_BUTTON (savedlg.radio_crlf), "CR");
+  savedlg.radio_lf = gtk_radio_button_new_with_label_from_widget
+                    (GTK_RADIO_BUTTON (savedlg.radio_cr), "LF");
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON
+                (mmaid->filetype == FILE_TYPE_DB ? savedlg.radio_db
+                                                 : savedlg.radio_plist), TRUE);
   switch (mmaid->playlist)
     {
-      case  1: gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radio3), TRUE);
-               break;
-      case  2: gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radio4), TRUE);
-               break;
-      default: gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radio2), TRUE);
+      case PLAYLIST_RELATIVE:
+        gtk_toggle_button_set_active
+                            (GTK_TOGGLE_BUTTON (savedlg.radio_relative), TRUE);
+        break;
+      case PLAYLIST_ABSOLUTE:
+        gtk_toggle_button_set_active
+                            (GTK_TOGGLE_BUTTON (savedlg.radio_absolute), TRUE);
+        break;
+      default:
+        gtk_toggle_button_set_active
+                            (GTK_TOGGLE_BUTTON (savedlg.radio_basename), TRUE);
     }
-  g_signal_connect (G_OBJECT (radio0), "toggled",
-                            G_CALLBACK (command_save_toggled_radio0), hbox0);
-  g_signal_connect (G_OBJECT (radio1), "toggled",
-                            G_CALLBACK (command_save_toggled_radio1), hbox0);
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON
+                        (mmaid->separator == '/' ? savedlg.radio_slash
+                                                 : savedlg.radio_yen), TRUE);
+  switch (mmaid->crlf)
+    {
+      case CODE_CRLF:
+        gtk_toggle_button_set_active
+                                (GTK_TOGGLE_BUTTON (savedlg.radio_crlf), TRUE);
+        break;
+      case CODE_CR:
+        gtk_toggle_button_set_active
+                                (GTK_TOGGLE_BUTTON (savedlg.radio_cr), TRUE);
+        break;
+      default:
+        gtk_toggle_button_set_active
+                                (GTK_TOGGLE_BUTTON (savedlg.radio_lf), TRUE);
+    }
+  g_signal_connect (G_OBJECT (savedlg.radio_db), "toggled",
+                G_CALLBACK (command_save_toggled_radio_db), &savedlg);
+  g_signal_connect (G_OBJECT (savedlg.radio_basename), "toggled",
+                G_CALLBACK (command_save_toggled_radio_basename), &savedlg);
+  /* ja:文字符号化方式 */
+  savedlg.combo_encoding = orz_charset_new ();
+  orz_charset_add_candidate (ORZ_CHARSET (savedlg.combo_encoding),
+                                                                charset_list);
+  orz_charset_set_charset (ORZ_CHARSET (savedlg.combo_encoding),
+                                                            mmaid->charset);
+  g_signal_connect (G_OBJECT (savedlg.combo_encoding), "edited",
+                                    G_CALLBACK (command_save_edited), dialog);
   /* ja:コンボボックス */
 #if GTK_CHECK_VERSION(2,24,0)
-  combo0 = gtk_combo_box_text_new ();
+  savedlg.combo_root = gtk_combo_box_text_new ();
   for (gl = g_list_first (glist); gl; gl = g_list_next (gl))
     {
-      gtk_combo_box_text_prepend_text (GTK_COMBO_BOX_TEXT (combo0), gl->data);
+      gtk_combo_box_text_prepend_text (GTK_COMBO_BOX_TEXT (savedlg.combo_root),
+                                                                    gl->data);
       if (g_strfilecmp (gl->data, root) == 0)
-        gtk_combo_box_set_active (GTK_COMBO_BOX (combo0), 0);
+        gtk_combo_box_set_active (GTK_COMBO_BOX (savedlg.combo_root), 0);
     }
-  g_signal_connect (G_OBJECT (combo0), "changed",
+  g_signal_connect (G_OBJECT (savedlg.combo_root), "changed",
                                     G_CALLBACK (command_save_changed), dialog);
 #elif GTK_CHECK_VERSION(2,4,0)
-  combo0 = gtk_combo_box_new_text ();
+  savedlg.combo_root = gtk_combo_box_new_text ();
   for (gl = g_list_first (glist); gl; gl = g_list_next (gl))
     {
-      gtk_combo_box_prepend_text (GTK_COMBO_BOX (combo0), gl->data);
+      gtk_combo_box_prepend_text (GTK_COMBO_BOX (savedlg.combo_root),
+                                                                    gl->data);
       if (g_strfilecmp (gl->data, root) == 0)
-        gtk_combo_box_set_active (GTK_COMBO_BOX (combo0), 0);
+        gtk_combo_box_set_active (GTK_COMBO_BOX (savedlg.combo_root), 0);
     }
-  g_signal_connect (G_OBJECT (combo0), "changed",
+  g_signal_connect (G_OBJECT (savedlg.combo_root), "changed",
                                     G_CALLBACK (command_save_changed), dialog);
 #else /* not GTK_CHECK_VERSION(2,4,0) */
-  combo0 = gtk_combo_new ();
-  gtk_combo_set_popdown_strings (GTK_COMBO (combo0), glist);
-  gtk_combo_set_value_in_list (GTK_COMBO (combo0), FALSE, FALSE);
-  gtk_combo_set_case_sensitive (GTK_COMBO (combo0), TRUE);
-  gtk_editable_set_editable (GTK_EDITABLE (GTK_COMBO (combo0)->entry), FALSE);
-  gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (combo0)->entry), root);
-  g_signal_connect (G_OBJECT (GTK_COMBO (combo0)->entry), "changed",
-                                    G_CALLBACK (command_save_changed), dialog);
+  savedlg.combo_root = gtk_combo_new ();
+  gtk_combo_set_popdown_strings (GTK_COMBO (savedlg.combo_root), glist);
+  gtk_combo_set_value_in_list (GTK_COMBO (savedlg.combo_root), FALSE, FALSE);
+  gtk_combo_set_case_sensitive (GTK_COMBO (savedlg.combo_root), TRUE);
+  gtk_editable_set_editable
+                (GTK_EDITABLE (GTK_COMBO (savedlg.combo_root)->entry), FALSE);
+  gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (savedlg.combo_root)->entry), root);
+  g_signal_connect (G_OBJECT (GTK_COMBO (savedlg.combo_root)->entry),
+                        "changed", G_CALLBACK (command_save_changed), dialog);
 #endif /* not GTK_CHECK_VERSION(2,4,0) */
-  g_list_find_custom (glist, (GFunc)g_free, NULL);
+  g_list_foreach (glist, (GFunc)g_free, NULL);
   g_list_free (glist);
-  /* ja:文字符号化方式 */
-  combo1 = orz_charset_new ();
-  orz_charset_add_candidate (ORZ_CHARSET (combo1), charset_list);
-  orz_charset_set_charset (ORZ_CHARSET (combo1), mmaid->charset);
-  g_signal_connect (G_OBJECT (combo1), "edited",
-                                    G_CALLBACK (command_save_edited), dialog);
   /* ja:ラベル */
-  label0 = gtk_label_new_with_mnemonic (_("_Root"));
-  label1 = gtk_label_new_with_mnemonic (_("Character _Encoding"));
+  label_path = gtk_label_new_with_mnemonic (_("_Path"));
+  gtk_label_set_mnemonic_widget (GTK_LABEL (label_path),
+                                                    savedlg.radio_relative);
+  label_sep = gtk_label_new_with_mnemonic (_("_Separator"));
+  gtk_label_set_mnemonic_widget (GTK_LABEL (label_sep), savedlg.radio_slash);
+  label_encoding = gtk_label_new_with_mnemonic (_("Character _Encoding"));
+  gtk_label_set_mnemonic_widget (GTK_LABEL (label_encoding),
+                                                    savedlg.combo_encoding);
+  savedlg.label_root = gtk_label_new_with_mnemonic (_("_Root"));
 #if GTK_CHECK_VERSION(2,4,0)
-  gtk_label_set_mnemonic_widget (GTK_LABEL(label0), combo0);
+  gtk_label_set_mnemonic_widget (GTK_LABEL (savedlg.label_root),
+                                                        savedlg.combo_root);
 #else /* not GTK_CHECK_VERSION(2,4,0) */
-  gtk_label_set_mnemonic_widget (GTK_LABEL(label0), GTK_COMBO (combo0)->entry);
+  gtk_label_set_mnemonic_widget (GTK_LABEL (savedlg.label_root),
+                                        GTK_COMBO (savedlg.combo_root)->entry);
 #endif /* not GTK_CHECK_VERSION(2,4,0) */
-  gtk_label_set_mnemonic_widget (GTK_LABEL(label1), combo1);
   /* ja:フレームとボックス */
   vbox = gtk_vbox_new (FALSE, SPACING);
   gtk_container_set_border_width (GTK_CONTAINER (vbox), SPACING);
-  hbox1 = gtk_hbox_new (FALSE, 0);
-  gtk_box_pack_start (GTK_BOX (hbox1), radio0, FALSE, FALSE, 0);
-  gtk_box_pack_start (GTK_BOX (hbox1), radio1, FALSE, FALSE, 0);
-  gtk_box_pack_start (GTK_BOX (hbox0), gtk_label_new ("("), FALSE, FALSE, 0);
-  gtk_box_pack_start (GTK_BOX (hbox0), radio2, FALSE, FALSE, 0);
-  gtk_box_pack_start (GTK_BOX (hbox0), radio3, FALSE, FALSE, 0);
-  gtk_box_pack_start (GTK_BOX (hbox0), radio4, FALSE, FALSE, 0);
-  gtk_box_pack_start (GTK_BOX (hbox0), gtk_label_new (")"), FALSE, FALSE, 0);
-  gtk_box_pack_start (GTK_BOX (hbox1), hbox0, FALSE, FALSE, 0);
-  gtk_box_pack_start (GTK_BOX (vbox), hbox1, TRUE, TRUE, 0);
-  hbox0 = gtk_hbox_new (FALSE, 0);
-  gtk_box_pack_start (GTK_BOX (hbox0), label0, FALSE, FALSE, 0);
-  gtk_box_pack_start (GTK_BOX (hbox0), combo0, TRUE, TRUE, 0);
-  gtk_box_pack_start (GTK_BOX (vbox), hbox0, TRUE, TRUE, 0);
-  hbox0 = gtk_hbox_new (FALSE, 0);
-  gtk_box_pack_start (GTK_BOX (hbox0), label1, FALSE, FALSE, 0);
-  gtk_box_pack_start (GTK_BOX (hbox0), combo1, TRUE, TRUE, 0);
-  gtk_box_pack_start (GTK_BOX (vbox), hbox0, TRUE, TRUE, 0);
+  savedlg.frame_path = gtk_frame_new (NULL);
+  gtk_frame_set_label_widget (GTK_FRAME (savedlg.frame_path), label_path);
+  hbox = gtk_hbox_new (FALSE, SPACING);
+  gtk_container_set_border_width (GTK_CONTAINER (hbox), SPACING);
+  gtk_box_pack_start (GTK_BOX (hbox), savedlg.radio_relative, FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (hbox), savedlg.radio_absolute, FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (hbox), savedlg.radio_basename, FALSE, FALSE, 0);
+  gtk_container_add (GTK_CONTAINER (savedlg.frame_path), hbox);
+  savedlg.frame_sep = gtk_frame_new (NULL);
+  gtk_frame_set_label_widget (GTK_FRAME (savedlg.frame_sep), label_sep);
+  hbox = gtk_hbox_new (FALSE, SPACING);
+  gtk_container_set_border_width (GTK_CONTAINER (hbox), SPACING);
+  gtk_box_pack_start (GTK_BOX (hbox), savedlg.radio_slash, FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (hbox), savedlg.radio_yen, FALSE, FALSE, 0);
+  gtk_container_add (GTK_CONTAINER (savedlg.frame_sep), hbox);
+  hbox = gtk_hbox_new (FALSE, SPACING);
+  gtk_box_pack_start (GTK_BOX (hbox), savedlg.radio_db, FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (hbox), savedlg.radio_plist, FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (hbox), savedlg.frame_path, FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (hbox), savedlg.frame_sep, FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (hbox), savedlg.check_header, FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 0);
+  frame = gtk_frame_new (NULL);
+  gtk_frame_set_label_widget (GTK_FRAME (frame), label_encoding);
+  hbox = gtk_hbox_new (FALSE, SPACING);
+  gtk_container_set_border_width (GTK_CONTAINER (hbox), SPACING);
+  gtk_box_pack_start (GTK_BOX (hbox), savedlg.combo_encoding, TRUE, TRUE, 0);
+  gtk_box_pack_start (GTK_BOX (hbox), savedlg.radio_crlf, FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (hbox), savedlg.radio_cr, FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (hbox), savedlg.radio_lf, FALSE, FALSE, 0);
+  gtk_container_add (GTK_CONTAINER (frame), hbox);
+  gtk_box_pack_start (GTK_BOX (vbox), frame, TRUE, TRUE, 0);
+  hbox = gtk_hbox_new (FALSE, SPACING);
+  gtk_box_pack_start (GTK_BOX (hbox), savedlg.label_root, FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (hbox), savedlg.combo_root, TRUE, TRUE, 0);
+  gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 0);
+  /* ja:表示 */
+  command_save_toggled_radio_db (savedlg.radio_db, &savedlg);
   gtk_widget_show_all (vbox);
   /* ja:ダイアログ */
 #if GTK_CHECK_VERSION(2,4,0)
   gtk_file_chooser_set_extra_widget (GTK_FILE_CHOOSER (dialog), vbox);
-  if (root)
-    gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dialog), root);
-  name = g_path_get_basename (orz_mdi_get_file (ORZ_MDI (mdi), page_num));
-  tmp = g_filename_to_utf8 (name, -1, NULL, NULL, NULL);
-  gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (dialog), tmp);
-  g_free (root);
-  g_free (tmp);
-  g_free (name);
+  if (save_path)
+    gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dialog), save_path);
   if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT)
     {
 # if ! GTK_CHECK_VERSION(2,6,0)
@@ -481,60 +568,73 @@ command_saveas (GtkWidget *widget,
 
       file = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
 # if GTK_CHECK_VERSION(2,24,0)
-      root = gtk_combo_box_text_get_active_text (GTK_COMBO_BOX_TEXT (combo0));
+      root = gtk_combo_box_text_get_active_text
+                                    (GTK_COMBO_BOX_TEXT (savedlg.combo_root));
 # elif GTK_CHECK_VERSION(2,6,0)
-      root = gtk_combo_box_get_active_text (GTK_COMBO_BOX (combo0));
+      root = gtk_combo_box_get_active_text
+                                        (GTK_COMBO_BOX (savedlg.combo_root));
 # else /* not GTK_CHECK_VERSION(2,6,0) */
-      if (gtk_combo_box_get_active_iter (GTK_COMBO_BOX (combo0), &iter))
-        gtk_tree_model_get (gtk_combo_box_get_model (GTK_COMBO_BOX (combo0)),
-                                                        &iter, 0, &root, -1);
+      if (gtk_combo_box_get_active_iter
+                                (GTK_COMBO_BOX (savedlg.combo_root), &iter))
+        gtk_tree_model_get (gtk_combo_box_get_model
+                    (GTK_COMBO_BOX (savedlg.combo_root)), &iter, 0, &root, -1);
 # endif /* not GTK_CHECK_VERSION(2,6,0) */
-      charset = orz_charset_get_charset (ORZ_CHARSET (combo1));
-      filekind = gtk_toggle_button_get_active
-                (GTK_TOGGLE_BUTTON (radio0)) ? FILE_KIND_DB : FILE_KIND_LIST;
-      if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (radio3)))
-        playlist = 1;
-      else if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (radio4)))
-        playlist = 2;
-      else
-        playlist = 0;
-    }
 #else /* not GTK_CHECK_VERSION(2,4,0) */
   gtk_box_pack_end (GTK_BOX (GTK_FILE_SELECTION (dialog)->main_vbox),
                                                         vbox, FALSE, FALSE, 0);
-  name = g_path_get_basename (orz_mdi_get_file (ORZ_MDI (mdi), page_num));
-  tmp = root ? g_strconcat (root, name, NULL)
-             : g_build_filename (save_path, name, NULL);
-  gtk_file_selection_set_filename (GTK_FILE_SELECTION (dialog), tmp);
-  g_free (root);
-  g_free (tmp);
-  g_free (name);
+  if (save_path)
+    {
+      path = g_strconcat (save_path, G_SEARCHPATH_SEPARATOR_S, NULL);
+      gtk_file_selection_set_filename (GTK_FILE_SELECTION (dialog), path);
+      g_free (path);
+    }
   gtk_file_selection_hide_fileop_buttons (GTK_FILE_SELECTION (dialog));
   if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_OK)
     {
       file = g_strdup (gtk_file_selection_get_filename
                                                 (GTK_FILE_SELECTION (dialog)));
       root = g_strdup (gtk_entry_get_text
-                                    (GTK_ENTRY (GTK_COMBO (combo0)->entry)));
-      charset = orz_charset_get_charset (ORZ_CHARSET (combo1));
-      filekind = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (radio0))
-                                            ? FILE_KIND_DB : FILE_KIND_LIST;
-      if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (radio3)))
-        playlist = 1;
-      else if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (radio4)))
-        playlist = 2;
-      else
-        playlist = 0;
-    }
+                        (GTK_ENTRY (GTK_COMBO (savedlg.combo_root)->entry)));
 #endif /* not GTK_CHECK_VERSION(2,4,0) */
+      charset = orz_charset_get_charset (ORZ_CHARSET (savedlg.combo_encoding));
+      filetype = gtk_toggle_button_get_active
+        (GTK_TOGGLE_BUTTON (savedlg.radio_db)) ? FILE_TYPE_DB : FILE_TYPE_LIST;
+      if (gtk_toggle_button_get_active
+                                (GTK_TOGGLE_BUTTON (savedlg.radio_relative)))
+        playlist = PLAYLIST_RELATIVE;
+      else if (gtk_toggle_button_get_active
+                                (GTK_TOGGLE_BUTTON (savedlg.radio_absolute)))
+        playlist = PLAYLIST_ABSOLUTE;
+      else
+        playlist = PLAYLIST_BASENAME;
+      separator = gtk_toggle_button_get_active
+                        (GTK_TOGGLE_BUTTON (savedlg.radio_slash)) ? '/' : '\\';
+      if (gtk_toggle_button_get_active
+                                    (GTK_TOGGLE_BUTTON (savedlg.radio_crlf)))
+        crlf = CODE_CRLF;
+      else if (gtk_toggle_button_get_active
+                                    (GTK_TOGGLE_BUTTON (savedlg.radio_cr)))
+        crlf = CODE_CR;
+      else
+        crlf = CODE_LF;
+      header = gtk_toggle_button_get_active
+                                    (GTK_TOGGLE_BUTTON (savedlg.check_header));
+    }
   else
     {
       file = root = charset = NULL;
-      filekind = FILE_KIND_DB;
-      playlist = 0;
+      filetype = FILE_TYPE_DB;
+      playlist = PLAYLIST_RELATIVE;
+      separator = G_DIR_SEPARATOR;
+#ifdef G_OS_WIN32
+      crlf = CODE_CRLF;
+#else /* not G_OS_WIN32 */
+      crlf = CODE_LF;
+#endif /* not G_OS_WIN32 */
+      header = TRUE;
     }
   gtk_widget_destroy (dialog);
-  if (file)
+  if (file && root && charset)
     {
       gboolean result = TRUE;
 
@@ -553,12 +653,11 @@ command_saveas (GtkWidget *widget,
           result = gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_YES;
           gtk_widget_destroy (dialog);
         }
-      if (result && (filekind == FILE_KIND_DB
-                    ? file_save_db (file, mmaid, root, charset)
-                    : file_save_list (file, mmaid, root, charset, playlist)))
+      if (result && (filetype == FILE_TYPE_DB
+                                    ? file_save_db (file, mmaid, root, charset)
+                                    : file_save_list (file, mmaid, charset,
+                                        playlist, separator, crlf, header)))
         {
-          gchar *path;
-
           g_free (save_path);
           path = fileio_get_full_path (file);
           save_path = g_path_get_dirname (path);
@@ -566,22 +665,27 @@ command_saveas (GtkWidget *widget,
           /* ja:キャラクターセット */
           g_free (mmaid->charset);
           mmaid->charset = g_strdup (charset);
-          if (filekind == FILE_KIND_DB || playlist != 2)
+          if (filetype == FILE_TYPE_DB || playlist != PLAYLIST_BASENAME)
             {
               /* ja:諸設定 */
               orz_mdi_set_created (ORZ_MDI (mdi), page_num, FALSE);
               orz_mdi_set_edited (ORZ_MDI (mdi), page_num, FALSE);
               orz_mdi_set_file (ORZ_MDI (mdi), page_num, file);
             }
-          mmaid->filekind = filekind;
-          if (filekind == FILE_KIND_LIST)
-            playlist = mmaid->playlist = playlist;
+          mmaid->filetype = filetype;
+          if (filetype == FILE_TYPE_LIST)
+            {
+              def_plist = mmaid->playlist = playlist;
+              def_sep = mmaid->separator = separator;
+              def_crlf = mmaid->crlf = crlf;
+              def_head = mmaid->header = header;
+            }
           orz_history_add_file (ORZ_HISTORY (history), file);
         }
-      g_free (file);
-      g_free (root);
-      g_free (charset);
     }
+  g_free (file);
+  g_free (root);
+  g_free (charset);
 }
 
 
@@ -665,11 +769,11 @@ command_export (GtkWidget *widget,
           file = g_filename_from_utf8 (id3tag->file, -1, NULL, NULL, NULL);
           switch (file_is_type (file))
             {
-              case FILE_KIND_ERROR:
+              case FILE_TYPE_ERROR:
                 /* ja:ファイルが存在しないとき */
                 str = _("File Open Error");
                 break;
-              case FILE_KIND_TAG:
+              case FILE_TYPE_TAG:
                 if (expinfo->hold)
                   {
                     /* ja:既存のタグを保持し、かつ既にタグがあるとき */
@@ -913,7 +1017,7 @@ command_refresh (GtkWidget *widget,
           file = g_filename_from_utf8 (id3tag0->file, -1, NULL, NULL, NULL);
           switch (file_is_type (file))
             {
-              case FILE_KIND_UNKNOWN:
+              case FILE_TYPE_MUSIC:
                 id3tag1 = file_open_id3tag (file, id3tag0->charset
                                             ? id3tag0->charset : charset_list);
                 n = mm_list_append (MM_LIST (MM_DIALOG (dialog)->list),
@@ -922,7 +1026,7 @@ command_refresh (GtkWidget *widget,
                   mm_list_show_refresh (MM_LIST (MM_DIALOG (dialog)->list), n);
                 mm_list_show_delete (MM_LIST (MM_DIALOG (dialog)->list), n);
                 break;
-              case FILE_KIND_TAG:
+              case FILE_TYPE_TAG:
                 id3tag1 = file_open_id3tag (file, id3tag0->charset
                                             ? id3tag0->charset : charset_list);
                 n = mm_list_append (MM_LIST (MM_DIALOG (dialog)->list),
@@ -1024,7 +1128,7 @@ command_search_recursive (const gchar *path,
                                 : NULL;
               switch (file_is_type (file))
                 {
-                  case FILE_KIND_UNKNOWN:
+                  case FILE_TYPE_MUSIC:
                     id3tag1 = file_open_id3tag (file, charset_list);
                     n1 = mm_list_append (MM_LIST (MM_DIALOG (dialog)->list),
                                                                     id3tag1);
@@ -1046,7 +1150,7 @@ command_search_recursive (const gchar *path,
                                 (MM_LIST (MM_DIALOG (dialog)->list), n1, TRUE);
                       }
                     break;
-                  case FILE_KIND_TAG:
+                  case FILE_TYPE_TAG:
                     id3tag1 = file_open_id3tag (file, charset_list);
                     n1 = mm_list_append (MM_LIST (MM_DIALOG (dialog)->list),
                                                                     id3tag1);
