@@ -68,31 +68,36 @@ const static gchar irivdb_header1[] = {
 
 
 /*  ja:パスのキャラクターセットを変換する
-       file,ファイル名
-     length,バイト数
-    charset,キャラクターセット
-        err,TRUE:エラー中断,FALSE:エラー強行
-        RET,変換されたファイル名,NULL:エラー                                */
+         file,ファイル名
+       length,バイト数
+      charset,キャラクターセット
+          err,TRUE:エラー中断,FALSE:エラー強行
+    separator,パスの区切り
+          RET,変換されたファイル名,NULL:エラー                              */
 static gchar *
 file_path_convert (const gchar    *file,
                    const gsize     length,
                    const gchar    *charset,
-                   const gboolean  err)
+                   const gboolean  err,
+                   gint           *separator)
 {
   gchar *result = NULL;
   gsize bytes_done = 0, leng;
+  gint sep = '\0';
 
   if (!file || length == 0 || !charset)
     return NULL;
-  leng = MIN (length == (gsize)-1 ? g_strlen (file) : length, 260);
+  leng = (gssize)length < 0 ? g_strlen (file) : length;
   while (bytes_done < leng)
     {
       gchar *tmp;
 
-      if (file[bytes_done] == '\\')
+      if (file[bytes_done] == '/' || file[bytes_done] == '\\')
         {
           tmp = result ? g_strconcat (result, G_DIR_SEPARATOR_S, NULL)
                        : g_strdup (G_DIR_SEPARATOR_S);
+          if (sep == '\0')
+            sep = file[bytes_done];
           bytes_done++;
         }
       else
@@ -127,32 +132,34 @@ file_path_convert (const gchar    *file,
       g_free (result);
       result = tmp;
     }
+  if (result && separator)
+    *separator = sep != '\0' ? sep : G_DIR_SEPARATOR;
   return result;
 }
 
 
 /*  ja:ファイルの種類を取得する
     file,ファイル名
-     RET,FILE_KIND_TAG:ID3TAG,FILE_KIND_DB:db,FILE_KIND_LIST:プレイリスト   */
+     RET,FILE_TYPE_TAG:ID3TAG,FILE_TYPE_DB:db,FILE_TYPE_LIST:プレイリスト   */
 guint
 file_is_type (const gchar *file)
 {
   gchar *buf = NULL, *dir, **ary, **charset;
   guint8 header[0xa0c0], tag[128];
-  gint i;
+  gint i, j = 0;
   gssize fsize;
   FileIO *fio;
 
   fio = fileio_open (file, FILEIO_ACCESS_READ, FILEIO_SHARE_READ,
                                                     FILEIO_MODE_OPEN_EXISTING);
   if (!fio)
-    return FILE_KIND_ERROR;
+    return FILE_TYPE_ERROR;
   if (fileio_read (fio, header, sizeof (header)) == sizeof (header)
     && g_memcmp (irivdb_header0, header, sizeof (irivdb_header0)) == 0
     && g_memcmp (irivdb_header1, header + 0xa080, sizeof (irivdb_header1))== 0)
     {
       fileio_close (fio);
-      return FILE_KIND_DB;
+      return FILE_TYPE_DB;
     }
   if (fileio_seek (fio, -128, FILEIO_SEEK_END) != -1
                     && fileio_read (fio, tag, sizeof (tag)) == sizeof (tag)
@@ -161,7 +168,7 @@ file_is_type (const gchar *file)
     {
       g_free (buf);
       fileio_close (fio);
-      return FILE_KIND_TAG;
+      return FILE_TYPE_TAG;
     }
   fsize = fileio_seek (fio, 0, FILEIO_SEEK_END);
   if (fsize < 0 || !(buf = g_malloc0 (fsize + sizeof (gchar)))
@@ -170,34 +177,54 @@ file_is_type (const gchar *file)
     {
       g_free (buf);
       fileio_close (fio);
-      return  FILE_KIND_UNKNOWN;
+      return FILE_TYPE_MUSIC;
     }
   fileio_close (fio);
+  /* ja:CRLFをLFに変換する */
+  for (i = 0; buf[i] != '\0'; i++)
+    if (buf[i] != '\r' || buf[i + 1] != '\n')
+      buf[j++] = buf[i];
+  buf[j] = '\0';
+  /* ja:CRをLFに変換する */
+  for (i = 0; buf[i] != '\0'; i++)
+    if (buf[i] == '\r')
+      buf[i] = '\n';
   /* ja:キャラクターセット変換 */
-  dir = g_path_get_dirname (file);
-  ary = g_strsplit (buf, "\r\n", 0);
-  charset = g_strsplit (charset_list, ",", 0);
+  ary = g_strsplit (buf, "\n", 0);
   g_free (buf);
+  for (i = 0; ary[i]; i++)
+    g_strstrip (ary[i]);
+  /* ja:ヘッダ */
+  for (i = 0; ary[i]
+        && (*(ary[i]) == '\0' || *(ary[i]) == '#' || *(ary[i]) == ';'); i++)
+    if (g_ascii_strcasecmp (ary[i], "#EXTM3U") == 0)
+      {
+        g_strfreev (ary);
+        return FILE_TYPE_LIST;
+      }
+  dir = g_path_get_dirname (file);
+  charset = g_strsplit (charset_list, ",", 0);
   for (i = 0; charset[i]; i++)
     {
-      gint j;
-
       for (j = 0; ary[j]; j++)
         if (*(ary[j]) != '\0' && *(ary[j]) != '#' && *(ary[j]) != ';')
           {
             gboolean result;
-            gchar *str;
+            gchar *path;
 
-            str = file_path_convert (ary[j], -1, charset[i], TRUE);
-            if (str)
+            path = file_path_convert (ary[j], -1, charset[i], TRUE, NULL);
+            if (path)
               {
-                gchar *path;
+                if (!g_path_is_absolute (path))
+                  {
+                    gchar *tmp;
 
-                path = g_path_is_absolute (str) ? g_strdup (str)
-                                        : g_build_filename (dir, str, NULL);
+                    tmp = g_build_filename (dir, path, NULL);
+                    g_free (path);
+                    path = tmp;
+                  }
                 result = g_file_test (path, G_FILE_TEST_EXISTS);
                 g_free (path);
-                g_free (str);
               }
             else
               {
@@ -211,13 +238,13 @@ file_is_type (const gchar *file)
           g_strfreev (charset);
           g_strfreev (ary);
           g_free (dir);
-          return  FILE_KIND_LIST;
+          return FILE_TYPE_LIST;
         }
     }
   g_strfreev (charset);
-  g_strfreev (ary);
   g_free (dir);
-  return  FILE_KIND_UNKNOWN;
+  g_strfreev (ary);
+  return FILE_TYPE_MUSIC;
 }
 
 
@@ -456,7 +483,7 @@ file_open_db (const gchar *file,
           while (gtk_events_pending ())
             gtk_main_iteration ();
           str = file_path_convert (mi_r[j].str[0], mi_r[j].len[0] - 1,
-                                                            charset[i], TRUE);
+                                                    charset[i], TRUE, NULL);
           if (!str)
             break;
           mi_c[j].str[0] = str;
@@ -490,7 +517,7 @@ file_open_db (const gchar *file,
             gtk_main_iteration ();
           mi_c[i].str[0]
                     = file_path_convert (mi_r[i].str[0], mi_r[i].len[0] - 1,
-                                                            charset[0], FALSE);
+                                                    charset[0], FALSE, NULL);
           if (!mi_c[i].str[0])
             break;
           for (j = 1; j < 5; j++)
@@ -582,7 +609,11 @@ file_open_list (const gchar *file,
                 MmaidWindow *mmaid)
 {
   gchar *buf = NULL, *dir, **ary, **charset;
-  gint i, plist = 0;
+  gint i, j = 0;
+  gint playlist = PLAYLIST_RELATIVE;
+  gint separator = G_DIR_SEPARATOR;
+  gint crlf = CODE_LF;
+  gboolean header = FALSE;
   gssize fsize;
   FileIO *fio;
   GList *glist = NULL;
@@ -608,59 +639,80 @@ file_open_list (const gchar *file,
       return;
     }
   fileio_close (fio);
+  /* ja:CRLFをLFに変換する */
+  for (i = 0; buf[i] != '\0'; i++)
+    if (buf[i] != '\r' || buf[i + 1] != '\n')
+      buf[j++] = buf[i];
+    else
+      crlf = CODE_CRLF;
+  buf[j] = '\0';
+  /* ja:CRをLFに変換する */
+  for (i = 0; buf[i] != '\0'; i++)
+    if (buf[i] == '\r')
+      {
+        buf[i] = '\n';
+        crlf = CODE_CR;
+      }
   /* ja:キャラクターセット変換 */
+  ary = g_strsplit (buf, "\n", 0);
+  g_free (buf);
+  for (i = 0; ary[i]; i++)
+    g_strstrip (ary[i]);
+  /* ja:ヘッダ */
+  for (i = 0; ary[i]
+        && (*(ary[i]) == '\0' || *(ary[i]) == '#' || *(ary[i]) == ';'); i++)
+    if (g_ascii_strcasecmp (ary[i], "#EXTM3U") == 0)
+      header = TRUE;
   dir = g_path_get_dirname (file);
-  ary = g_strsplit (buf, "\r\n", 0);
   charset = g_strsplit (mmaid->charset, ",", 0);
   g_free (mmaid->charset);
-  g_free (buf);
   for (i = 0; charset[i]; i++)
     {
       gint j;
 
-      plist = 0;
+      playlist = PLAYLIST_RELATIVE;
       for (j = 0; ary[j]; j++)
         if (*(ary[j]) != '\0' && *(ary[j]) != '#' && *(ary[j]) != ';')
           {
-            gchar *str, *path;
+            gchar *path;
 
             while (gtk_events_pending ())
               gtk_main_iteration ();
-            str = file_path_convert (ary[j], -1, charset[i], TRUE);
-            if (!str)
+            path = file_path_convert (ary[j], -1,
+                                                charset[i], TRUE, &separator);
+            if (!path)
               break;
-            if (g_path_is_absolute (str))
+            if (g_path_is_absolute (path))
               {
-                path = g_strdup (str);
-                plist = 1;
-              }
-            else
-              {
-                path = g_build_filename (dir, str, NULL);
-              }
-            g_free (str);
-            if (g_file_test (path, G_FILE_TEST_EXISTS))
-              {
-                glist = g_list_append (glist, path);
+                playlist = PLAYLIST_ABSOLUTE;
               }
             else
               {
+                gchar *tmp;
+
+                tmp = g_build_filename (dir, path, NULL);
                 g_free (path);
-                break;
+                path = tmp;
               }
+            glist = g_list_append (glist, path);
           }
       if (!ary[j])
         break;
-      g_list_find_custom (glist, (GFunc)g_free, NULL);
+      g_list_foreach (glist, (GFunc)g_free, NULL);
       g_list_free (glist);
       glist = NULL;
     }
   mmaid->charset = g_strdup (charset[i] ? charset[i] : charset[0]);
   g_strfreev (charset);
-  g_strfreev (ary);
   g_free (dir);
+  g_strfreev (ary);
   if (g_list_length (glist) > 0)
-    mmaid->playlist = plist;
+    {
+      mmaid->playlist = playlist;
+      mmaid->separator = separator;
+      mmaid->crlf = crlf;
+      mmaid->header = header;
+    }
   for (glist = g_list_first (glist);
                             glist; glist = g_list_delete_link (glist, glist))
     {
@@ -692,7 +744,10 @@ file_open_edit (const gchar *file)
   GtkWidget *scroll;
 
   mmaid = g_malloc0 (sizeof (MmaidWindow));
-  mmaid->playlist = playlist;
+  mmaid->playlist = def_plist;
+  mmaid->separator = def_sep;
+  mmaid->crlf = def_crlf;
+  mmaid->header = def_head;
   /* ja:リスト */
   mmaid->list = mm_list_new ();
   mm_list_set_editable (MM_LIST (mmaid->list), TRUE);
@@ -707,14 +762,14 @@ file_open_edit (const gchar *file)
       charset = g_strsplit (charset_list, ",", 0);
       mmaid->charset = g_strdup (charset[0]);
       g_strfreev (charset);
-      mmaid->filekind = FILE_KIND_DB;
+      mmaid->filetype = FILE_TYPE_DB;
     }
   else
     {
       filename = fileio_get_full_path (file);
       mmaid->charset = g_strdup (charset_list);
-      mmaid->filekind = file_is_type (file);
-      if (mmaid->filekind == FILE_KIND_LIST)
+      mmaid->filetype = file_is_type (file);
+      if (mmaid->filetype == FILE_TYPE_LIST)
         file_open_list (file, mmaid);
       else
         file_open_db (file, mmaid);
@@ -1082,36 +1137,75 @@ file_save_db (const gchar *file,
 
 
 /*  ja:プレイリストを保存する
-        file,ファイル名
-       mmaid,ウインドウ情報
-        root,ルート
-     charset,キャラクターセット
-    playlist.プレイリスト(0:相対,1:絶対,2:名前)
-         RET,TRUE:正常終了,FALSE:エラー                                     */
+         file,ファイル名
+        mmaid,ウインドウ情報
+      charset,キャラクターセット
+     playlist,プレイリスト
+    separator,パスの区切り
+         crlf,改行コード
+       header,ヘッダ
+          RET,TRUE:正常終了,FALSE:エラー                                    */
 gboolean
-file_save_list (const gchar *file,
-                MmaidWindow *mmaid,
-                const gchar *root,
-                const gchar *charset,
-                const gint   playlist)
+file_save_list (const gchar    *file,
+                MmaidWindow    *mmaid,
+                const gchar    *charset,
+                const gint      playlist,
+                const gint      separater,
+                const gint      crlf,
+                const gboolean  header)
 {
   FileIO *fio;
-  GtkWidget *dialog;
 
   fio = fileio_open (file, FILEIO_ACCESS_READ | FILEIO_ACCESS_WRITE,
                                 FILEIO_SHARE_READ, FILEIO_MODE_CREATE_ALWAYS);
   if (fio)
     {
+      char *root;
+      const char *sep, *newline;
       gint i, count;
+      GtkWidget *dialog;
 
+      sep = separater == '/' ? "/" : "\\";
+      switch (crlf)
+        {
+          case CODE_CRLF: newline = "\r\n"; break;
+          case CODE_CR:   newline = "\r"; break;
+          default:        newline = "\n";
+        }
+      root = g_path_get_dirname (file);
       /* ja:ダイアログボックス */
       userbreak = TRUE;
       dialog = abort_dialog (_("Writing"));
       gtk_grab_add (dialog);
+      /* ja:ヘッダ */
+      if (header)
+        {
+          gsize leng;
+          gchar *tmp;
+
+          tmp = g_strconcat ("#EXTM3U", newline, NULL);
+          leng = g_strlen (tmp) * sizeof (gchar);
+          if (fileio_write (fio, tmp, leng) != leng)
+            {
+              gtk_grab_remove (dialog);
+              gtk_widget_destroy (dialog);
+              dialog = gtk_message_dialog_new (GTK_WINDOW (window),
+                            GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+                            GTK_MESSAGE_WARNING, GTK_BUTTONS_OK,
+                            _("Can not write"));
+              g_signal_connect (G_OBJECT (dialog), "key-press-event",
+                                    G_CALLBACK (misc_dialog_key_press), NULL);
+              gtk_dialog_run (GTK_DIALOG (dialog));
+              gtk_widget_destroy (dialog);
+              dialog = NULL;
+              userbreak = FALSE;
+            }
+          g_free (tmp);
+        }
       count = mm_list_length (MM_LIST (mmaid->list));
       for (i = 0; i < count && userbreak; i++)
         {
-          gchar *element, *tmp, **ary;
+          gchar *element, **ary;
           gint j;
           const ID3Tag *id3tag;
 
@@ -1120,7 +1214,7 @@ file_save_list (const gchar *file,
           id3tag = mm_list_get_nth (MM_LIST (mmaid->list), i);
           switch (playlist)
             {
-              case 0:/* ja:相対パス */
+              case PLAYLIST_RELATIVE:/* ja:相対パス */
                 {
                   gchar *file, *relative;
 
@@ -1133,7 +1227,7 @@ file_save_list (const gchar *file,
                   g_free (relative);
                 }
                 break;
-              case 1:/* ja:絶対パス */
+              case PLAYLIST_ABSOLUTE:/* ja:絶対パス */
                 element = g_strdup (id3tag->file);
                 break;
               default:/* ja:ファイル名のみ */
@@ -1148,11 +1242,9 @@ file_save_list (const gchar *file,
                   g_free (name);
                 }
             }
-          tmp = g_strconcat (element, "\r\n", NULL);
+          ary = g_strsplit (element, G_DIR_SEPARATOR_S, 0);
           g_free (element);
           element = NULL;
-          ary = g_strsplit (tmp, G_DIR_SEPARATOR_S, 0);
-          g_free (tmp);
           for (j = 0; ary[j]; j++)
             {
               gchar *name;
@@ -1179,7 +1271,9 @@ file_save_list (const gchar *file,
                 }
               if (element)
                 {
-                  tmp = g_strconcat (element, "\\", name, NULL);
+                  gchar *tmp;
+
+                  tmp = g_strconcat (element, sep, name, NULL);
                   g_free (name);
                   g_free (element);
                   element = tmp;
@@ -1193,9 +1287,12 @@ file_save_list (const gchar *file,
           if (element)
             {
               gsize leng;
+              gchar *tmp;
 
-              leng = g_strlen (element) * sizeof (gchar);
-              if (fileio_write (fio, element, leng) != leng)
+              tmp = g_strconcat (element, newline, NULL);
+              g_free (element);
+              leng = g_strlen (tmp) * sizeof (gchar);
+              if (fileio_write (fio, tmp, leng) != leng)
                 {
                   gtk_grab_remove (dialog);
                   gtk_widget_destroy (dialog);
@@ -1210,7 +1307,7 @@ file_save_list (const gchar *file,
                   dialog = NULL;
                   userbreak = FALSE;
                 }
-              g_free (element);
+              g_free (tmp);
             }
         }
       /* ja:終了処理 */
@@ -1219,10 +1316,13 @@ file_save_list (const gchar *file,
           gtk_grab_remove (dialog);
           gtk_widget_destroy (dialog);
         }
+      g_free (root);
       fileio_close (fio);
     }
   else
     {
+      GtkWidget *dialog;
+
       dialog = gtk_message_dialog_new (GTK_WINDOW (window),
                             GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
                             GTK_MESSAGE_WARNING, GTK_BUTTONS_OK,
