@@ -1,6 +1,6 @@
 /*
     Melody maid
-    copyright (c) 1998-2010 Kazuki Iwamoto http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2012 Kazuki Iwamoto http://www.maid.org/ iwm@maid.org
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -51,7 +51,10 @@ keyfile_read (const gchar *file)
       n_pos = g_key_file_get_integer (key_file, "pref", "npos", NULL);
       def_hold = g_key_file_get_boolean (key_file, "mmaid", "defhold", NULL);
       def_gnum = g_key_file_get_integer (key_file, "mmaid", "defgnum", NULL);
-      playlist = g_key_file_get_integer (key_file, "mmaid", "playlist", NULL);
+      def_plist = g_key_file_get_integer (key_file, "mmaid", "playlist", NULL);
+      def_sep = g_key_file_get_integer (key_file, "mmaid", "separator", NULL);
+      def_crlf = g_key_file_get_integer (key_file, "mmaid", "crlf", NULL);
+      def_head = g_key_file_get_boolean (key_file, "mmaid", "header", NULL);
       cmd_play = g_key_file_get_string (key_file, "mmaid", "cmdplay", NULL);
       cmd_stop = g_key_file_get_string (key_file, "mmaid", "cmdstop", NULL);
       charset_list = g_key_file_get_string (key_file, "mmaid", "charsetlist",
@@ -187,7 +190,10 @@ keyfile_write (const gchar *file)
       g_key_file_set_integer (key_file, "pref", "npos", n_pos);
       g_key_file_set_boolean (key_file, "mmaid", "defhold", def_hold);
       g_key_file_set_integer (key_file, "mmaid", "defgnum", def_gnum);
-      g_key_file_set_integer (key_file, "mmaid", "playlist", playlist);
+      g_key_file_set_integer (key_file, "mmaid", "playlist", def_plist);
+      g_key_file_set_integer (key_file, "mmaid", "separator", def_sep);
+      g_key_file_set_integer (key_file, "mmaid", "crlf", def_crlf);
+      g_key_file_set_boolean (key_file, "mmaid", "header", def_head);
       g_key_file_set_string (key_file, "mmaid", "charsetlist", charset_list);
       if (cmd_play)
         g_key_file_set_string (key_file, "mmaid", "cmdplay", cmd_play);
