@@ -1,6 +1,6 @@
 /*
     Melody maid
-    copyright (c) 1998-2008 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2011 Kazuki Iwamoto http://www.maid.org/ iwm@maid.org
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -91,7 +91,8 @@ process_dialog (void)
   gtk_label_set_mnemonic_widget (GTK_LABEL (label0), entry0);
   gtk_label_set_mnemonic_widget (GTK_LABEL (label1), entry1);
   /* ja:フレームとボックス */
-  vbox = gtk_vbox_new (FALSE, SPACING);
+  vbox = gtk_dialog_get_content_area (GTK_DIALOG (dialog));
+  gtk_box_set_spacing (GTK_BOX (vbox), SPACING);
   gtk_container_set_border_width (GTK_CONTAINER (vbox), SPACING);
   hbox = gtk_hbox_new (FALSE, 0);
   gtk_box_pack_start (GTK_BOX (hbox), label0, FALSE, FALSE, 0);
@@ -101,7 +102,6 @@ process_dialog (void)
   gtk_box_pack_start (GTK_BOX (hbox), label1, FALSE, FALSE, 0);
   gtk_box_pack_start (GTK_BOX (hbox), entry1, TRUE, TRUE, 0);
   gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
-  gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dialog)->vbox), vbox);
 
   /* ja:表示 */
   gtk_dialog_set_response_sensitive (GTK_DIALOG (dialog), GTK_RESPONSE_OK,
