@@ -113,13 +113,13 @@ signal_drag_data_received (GtkWidget        *widget,
                 {
                   switch (file_is_type (file))
                     {
-                      case FILE_KIND_ERROR:
-                      case FILE_KIND_DB:
-                      case FILE_KIND_LIST:
+                      case FILE_TYPE_ERROR:
+                      case FILE_TYPE_DB:
+                      case FILE_TYPE_LIST:
                         mmaid = file_open_edit (file);
                         break;
-                      case FILE_KIND_UNKNOWN:
-                      case FILE_KIND_TAG:
+                      case FILE_TYPE_MUSIC:
+                      case FILE_TYPE_TAG:
                         {
                           ID3Tag *id3tag;
 
@@ -201,13 +201,13 @@ signal_selection_recv (GObject   *object,
     if (arg_files[i])
       switch (file_is_type (argv[i]))
         {
-          case FILE_KIND_ERROR:
-          case FILE_KIND_DB:
-          case FILE_KIND_LIST:
+          case FILE_TYPE_ERROR:
+          case FILE_TYPE_DB:
+          case FILE_TYPE_LIST:
             mmaid = file_open_edit (argv[i]);
             break;
-          case FILE_KIND_UNKNOWN:
-          case FILE_KIND_TAG:
+          case FILE_TYPE_MUSIC:
+          case FILE_TYPE_TAG:
             {
               ID3Tag *id3tag;
 
